/*	Sample program for Olimex AVR-MT with AT90S2313 processor
 *	Press Button1 to set the upline text
 *	Press Button2 to set upline and downline text
 *	Press Button5 to set the downline text
 *	Compile with AVRStudio+WinAVR (gcc version 3.4.6)
 */

#define __AVR_AT90S2313__	1

#include "avr/io.h"
#include "lcd.h"

//unsigned char Text, br;
const unsigned char Text1[] = "     AVR-MT     ";
const unsigned char Text2[] = " olimex.com/dev ";

unsigned int Time, T;
unsigned char i, B[7];

void Delay_Ms(int Temp)
{
	int Ms;
	while (Temp)
	{
		for (Ms = 560; Ms; Ms--);		//1ms is ~560 empty cycles
		Temp--;
	}
}

void PORT_Init()
{
	PORTB = 0b00000000;
	DDRB  = 0b00011111;		//set Buzzer as output (Bit4 = 1); set D1-D4 as output (Bit0-3 = 1)

	PORTD = 0b00000000;
	DDRD  = 0b01011010;		//set E as output (Bit6 = 0)
							//set RS as output(Bit4 = 0)
							//set Relay as output (Bit3 = 1)
							//set TX as output(Bit1 = 1)
}

void E_Pulse()
{
	unsigned int D;
	PORTD = PORTD | 0b01000000;	//set E to high
	for (D=56; D; D--);
	PORTD = PORTD & 0b10111111;	//set E to low
}

void LCD_Init()
{
	PORTD = PORTD & 0b11101111;	//set RS = 0
	Delay_Ms(110);

	PORTB = 0b00000011;
	E_Pulse();
	Delay_Ms(10);

	PORTB = 0b00000011;
	E_Pulse();
	Delay_Ms(10);

	PORTB = 0b00000011;
	E_Pulse();
	Delay_Ms(10);

	PORTB = 0b00000010;
	E_Pulse();
}

void LCDSendChar(unsigned char Temp)
{
	Delay_Ms(2);

	T = Temp>>4;					//set in T first 4 bits of Temp
	PORTB = (PORTB & 0b11110000) | T;
	PORTD = PORTD | 0b00010000;		//set RS = 1
	E_Pulse();

	T = Temp & 0b00001111;			//set in T last 4 bits of Temp
	PORTB = (PORTB & 0b11110000) | T;
	PORTD = PORTD | 0b00010000;		//set RS = 1
	E_Pulse();
}

void LCDSendCommand(unsigned char Temp)
{
	Delay_Ms(2);

	T = Temp>>4;					//set in T first 4 bits of Temp
	PORTB = (PORTB & 0b11110000) | T;
	PORTD = PORTD & 0b11101111;		//set RS = 0;
	E_Pulse();

	T = Temp & 0b00001111;			//set in T last 4 bits of Temp
	PORTB = (PORTB & 0b11110000) | T;
	PORTD = PORTD & 0b11101111;		//set RS = 0;
	E_Pulse();
}

/*
	Button state check:
	set one of the pins (PORTB5, PORTB6 or PORTB7) as output 
	with high value and watch states of other two pins 
	accordingly schematic as follow:
*/

//Button1: PORTB5 output, check PINB6
unsigned char Button_1()
{
	PORTB = (PORTB & 0b00011111) | 0b00100000;
	DDRB  = (DDRB  & 0b00011111) | 0b00100000;

	if (PINB & 0b01000000)
		return 1;
	else
		return 0;
}

//Button2: PORTB6 output, check PINB7
unsigned char Button_2()
{
	PORTB = (PORTB & 0b00011111) | 0b01000000;
	DDRB  = (DDRB  & 0b00011111) | 0b01000000;

	if (PINB & 0b10000000)
		return 1;
	else
		return 0;
}

//Button3: PORTB7 output, check PINB5
unsigned char Button_3()
{
	PORTB = (PORTB & 0b00011111) | 0b10000000;
	DDRB  = (DDRB  & 0b00011111) | 0b10000000;

	if (PINB & 0b00100000)
		return 1;
	else
		return 0;
}

//Button4: PORTB6 output, check PINB5
unsigned char Button_4()
{
	PORTB = (PORTB & 0b00011111) | 0b01000000;
	DDRB  = (DDRB  & 0b00011111) | 0b01000000;

	if (PINB & 0b00100000)
		return 1;
	else
		return 0;
}

//Button5: PORTB7 output, check PINB6
unsigned char Button_5()
{
	PORTB = (PORTB & 0b00011111) | 0b10000000;
	DDRB  = (DDRB  & 0b00011111) | 0b10000000;

	if (PINB & 0b01000000)
		return 1;
	else
		return 0;
}

//Button1: PORTB6 output, check PINB7
unsigned char Button_6()
{
	PORTB = (PORTB & 0b00011111) | 0b00100000;
	DDRB  = (DDRB  & 0b00011111) | 0b00100000;

	if (PINB & 0b10000000)
		return 1;
	else
		return 0;
}

int main()
{
	PORT_Init();
	LCD_Init();
	//value of the array B keep state of the buttons (0 - released; 1 - pressed)
	for (i=1; i<=6; i++) B[i] = 0;
	LCDSendCommand(DISP_ON);
	LCDSendCommand(CLR_DISP);
	while(1)
	{
		if (Button_1() & (!B[1]))
		{
			LCDSendCommand(CLR_DISP);
			LCDSendCommand(DD_RAM_ADDR);
			for (i=0; i<16; i++) LCDSendChar(Text1[i]);
			B[1] = 1;
		}

		if (!Button_1() & B[1]) B[1] = 0;

		if (Button_2() & (!B[2]))
		{
			LCDSendCommand(CLR_DISP);
			LCDSendCommand(DD_RAM_ADDR);
			for (i=0; i<16; i++) LCDSendChar(Text1[i]);
			LCDSendCommand(DD_RAM_ADDR2);
			for (i=0; i<16; i++) LCDSendChar(Text2[i]);
			B[2] = 1;
		}

		if (!Button_2() & B[2]) B[2] = 0;

		if (Button_5() & (!B[5]))
		{
			LCDSendCommand(CLR_DISP);
			LCDSendCommand(DD_RAM_ADDR2);
			for (i=0; i<16; i++) LCDSendChar(Text2[i]);
			B[5] = 1;
		}

		if (!Button_5() & B[5]) B[5] = 0;
	}
}
